import { select, 
        geoPath, 
        geoNaturalEarth1,
        geoCentroid,
        scaleSqrt,
        schemeRdYlGn,
        zoom,
        max,
        event,
       	format } from 'd3';
import { feature } from 'topojson';
import { loadAndProcessData } from './loadAndProcessData.js';
import { sizeLegend } from './sizeLegend.js';

const svg = select('svg');
const width = parseFloat(svg.attr('width'));
const height = +svg.attr('height');

const projection = geoNaturalEarth1().clipExtent([[0,0],[width,height]]);
const pathGenerator = geoPath().projection(projection);
const radiusValue = d => d.properties['2018']

const g = svg.append('g');

const colorLegendG = svg.append('g')
	.attr('transform', `translate(50,250)`)

g.append('path')
	.attr('class', 'sphere')
	.attr('d', pathGenerator({type:'Sphere'}))

svg.call(zoom().on('zoom', () => {
	g.attr('transform', event.transform)
}));

loadAndProcessData().then(countries => {
  
  const sizeScale = scaleSqrt()
    .domain([0, max(countries.features, radiusValue)])
  	.range([0, 33]);
  
  const paths = g.selectAll('path')
  	.data(countries.features);
  
  paths.enter().append('path')
  	.attr('class', 'country')
  	.attr('d', pathGenerator)
  	.attr('fill', d => d.properties['2018'] ? '#d8d8d8' : '#fec1c1')
  .append('title')
  	.text(d => isNaN(radiusValue(d))
          ? 'Missing Data'
          :[d.properties['Region, subregion, country or area *'], 
                format(',')(radiusValue(d))].join(': '));
  
  countries.featuresWithPopulation.forEach(d => {
  	d.properties.projected = projection(geoCentroid(d));
  })
  
  const circles = g.selectAll('circle')
  	.data(countries.featuresWithPopulation)
  
  circles.enter().append('circle')
  	.attr('class', 'country-circles')
  	.attr('cx', d => d.properties.projected[0])
    .attr('cy', d => d.properties.projected[1])
  	.attr('r', d => sizeScale(radiusValue(d)));
  
  g.append('g')
    .attr('transform', `translate(50,100)`)
    .call(sizeLegend, {
      sizeScale,
      spacing: 50,
      textOffset: 20,
      numTicks: 5,
      fillTicks: 'rgba(0,0,0,0.5)',
    	tickFormat: d3.format(',')
		})
  	.append('text')
  		.attr('class', 'legend-title')
  		.text('Population')
  		.attr('y', -50)
  		.attr('x', -30)  		
});